import bpy
from bpy.types import Context, Object, Operator, Panel, PoseBone, UILayout
# ---------------------------------------------------------
# use prefix '|CONTROL|' on bones you want to transfer.
# ---------------------------------------------------------

# ---------------------------------------------------------
CONTROL_PREFIX = '|CONTROL|'
# ---------------------------------------------------------

def add_bone_collection(to_armature, name):
    # tries adding a collection.
    try:
        col = to_armature.data.collections_all[name]
    except:
        to_armature.data.collections.new(name)

def add_external_rig(from_armature, to_armature, parent_bone):
    
    # prepare things for script
    bpy.context.view_layer.objects.active = to_armature
    bpy.ops.object.mode_set(mode='EDIT')

    # create new bones in to_armature
    for bone in from_armature.data.bones:
        if CONTROL_PREFIX in bone.name:
            print(bone.name)
            new_bone_name = bone.name.replace(CONTROL_PREFIX,'')
            print("  " + str(bone.head))
            new_bone = to_armature.data.edit_bones.new(new_bone_name)
            new_bone_parent = to_armature.data.edit_bones[parent_bone]
            new_bone.head = bone.head + from_armature.location
            new_bone.tail = bone.tail + from_armature.location
            new_bone.parent = new_bone_parent
    # add constraints to bones in from_armature
    for bone in from_armature.pose.bones:
        if CONTROL_PREFIX in bone.name:
            find_bone_name = bone.name.replace(CONTROL_PREFIX,'')
            constraint = bone.constraints.new(type='COPY_TRANSFORMS')
            constraint.target = to_armature
            constraint.subtarget = find_bone_name
            
    bpy.ops.object.mode_set(mode='OBJECT')
    
def add_external_bones(from_armature, to_armature, parent_bone, collection_name='External', use_rigify=False):
    # prepare things for script
    add_bone_collection(to_armature, collection_name)
    
    collection = to_armature.data.collections_all[collection_name]
    
    bpy.context.view_layer.objects.active = to_armature
    bpy.ops.object.mode_set(mode='EDIT')
    bone_list = []
    # create new bones in to_armature
    for bone in from_armature.data.bones:
        if CONTROL_PREFIX in bone.name:
            print(bone.name)
            new_bone_name = bone.name.replace(CONTROL_PREFIX,'')
            print("  " + str(bone.head))
            new_bone = to_armature.data.edit_bones.new(new_bone_name)
            new_bone_parent = to_armature.data.edit_bones[parent_bone]
            new_bone.head = bone.head + from_armature.location
            new_bone.tail = bone.tail + from_armature.location
            new_bone.parent = new_bone_parent
            collection.assign(new_bone)
            bone_list.append(new_bone.name)

    bpy.ops.object.mode_set(mode='OBJECT')
    
    # use rigify stuff if we're in rigify mode.
    if use_rigify:
        for bone in from_armature.data.bones:
            if CONTROL_PREFIX in bone.name:
                new_bone_name = bone.name.replace(CONTROL_PREFIX,'')
                pose_bone = to_armature.pose.bones[new_bone_name]
                pose_bone.rigify_type = 'basic.super_copy'
                pose_bone.rigify_parameters.make_deform = False
                pose_bone.rigify_parameters.super_copy_widget_type = 'bone'

def attach_rig(from_armature, to_armature, parent_bone):
    for bone in from_armature.pose.bones:
        if CONTROL_PREFIX in bone.name:
            find_bone_name = bone.name.replace(CONTROL_PREFIX,'')
            constraint = bone.constraints.new(type='COPY_TRANSFORMS')
            constraint.target = to_armature
            constraint.subtarget = find_bone_name
    bpy.ops.object.mode_set(mode='OBJECT')    






'''
to_armature = bpy.data.objects['metarig-pig']
collection_name = 'eyes'
collection = to_armature.data.collections_all[collection_name]
collection.assign(bpy.data.armatures["metarig-pig"].bones["lips anchor upper jaw pos.L"])
'''








    
bl_info = {
    "name": "External rig rigging helper",
    "author": "Velocirection (V-rex)",
    "version" : (0, 0, 0, 0),
    "blender" : (4, 1, 0),
    "location": "View3D - Properties",
    "description": "External rig rigging helper.",
    "warning": "",
    "wiki_url": "",
    "category": "rigging"}

class errh_add_bones_and_attach(bpy.types.Operator):
    bl_idname = 'scene.errh_add_bones_and_attach'
    bl_label = 'errh_add_bones_and_attach'
    bl_description = ''
    
    def execute(self, context):
        scene            = context.scene
        toArmature       = scene.errh_to_armature
        fromArmature     = scene.errh_from_armature
        toParentBoneName = scene.errh_parent_bone
        print(f'>errh_add_bones_and_attach called:')
        print(f'  >toAmature: {toArmature}')
        print(f'  >fromAmature: {fromArmature}')
        print(f'  >toParentBoneName: {toParentBoneName}')
        #add_external_rig(fromArmature, toArmature, toParentBoneName)
        add_external_bones(fromArmature, toArmature, toParentBoneName)
        attach_rig(fromArmature, toArmature, toParentBoneName)
        return {'FINISHED'}
    
class errh_add_bones_to_armature(bpy.types.Operator):
    bl_idname = 'scene.errh_add_bones_to_armature'
    bl_label = 'errh_add_bones_to_armature'
    bl_description = ''
    
    def execute(self, context):
        scene            = context.scene
        toArmature       = scene.errh_to_armature
        fromArmature     = scene.errh_from_armature
        toParentBoneName = scene.errh_parent_bone
        collectionName   = scene.errh_bone_collection_name
        useRigify = scene.errh_use_rigify_mode
        print(f'>errh_add_bones_to_armature called:')
        print(f'  >toAmature: {toArmature}')
        print(f'  >fromAmature: {fromArmature}')
        print(f'  >toParentBoneName: {toParentBoneName}')
        add_external_bones(fromArmature, toArmature, toParentBoneName, collectionName, useRigify)
        return {'FINISHED'}
    
class errh_attach_rig(bpy.types.Operator):
    bl_idname = 'scene.errh_attach_rig'
    bl_label = 'errh_attach_rig'
    bl_description = ''
    
    def execute(self, context):
        scene            = context.scene
        toArmature       = scene.errh_to_armature_generated
        fromArmature     = scene.errh_from_armature
        toParentBoneName = scene.errh_parent_bone
        print(f'>errh_attach_rig called:')
        print(f'  >toAmature: {toArmature}')
        print(f'  >fromAmature: {fromArmature}')
        print(f'  >toParentBoneName: {toParentBoneName}')
        attach_rig(fromArmature, toArmature, toParentBoneName)
        return {'FINISHED'}
    

# MAIN PANEL ---------------------------------------------------------------
class UI_PT_errh_panel(bpy.types.Panel):
    bl_label = 'External Rig Rigging Helper'
    bl_idname = 'UI_PT_errh_panel'
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'Vrextras'
    
    # modified functions from the copy global transform add on.
    def _bone_search(self, layout: UILayout, scene: bpy.types.Scene, armature_ob: bpy.types.Object) -> None:
        """Search within the bones of the given armature."""
        assert armature_ob and armature_ob.type == 'ARMATURE'

        layout.prop_search(
            scene,
            "errh_parent_bone",
            armature_ob.data,
            "edit_bones" if armature_ob.mode == 'EDIT' else "bones",
            text="Parent Bone",
    )

    def draw(self, context: Context) -> None:
        scene = context.scene
        using_rigify = scene.errh_use_rigify_mode
        layout = self.layout
        
        box = layout.box()
        split = box.split()
        col1 = split.column()
        col1.prop(scene, "errh_use_rigify_mode", text="Use rigify mode")
        

        # ----------------------------------------------------   
        
        # armatures/bone box ------------------------------
        box = layout.box()
        split = box.split()
        
        # from armature
        col1 = split.column()
        col1.label(text='From Armature')
        col1.prop(scene, 'errh_from_armature', text="")
        
        # to armature section ---------------------------------
        col2 = split.column()
        if using_rigify:
            col2.label(text='To Metarig')
            col2.prop(scene, 'errh_to_armature', text="") 
            toArmature    = scene.errh_to_armature
            fromArmature  = scene.errh_from_armature
            parentBone    = scene.errh_parent_bone
            row = box.row()
            row.prop(scene, 'errh_bone_collection_name', text="Collection")
            if toArmature != None and toArmature.type == 'ARMATURE':
                self._bone_search(box, scene, toArmature)
        else:
            col2.label(text='To Armature')
            col2.prop(scene, 'errh_to_armature', text="") 
            toArmature    = scene.errh_to_armature
            fromArmature  = scene.errh_from_armature
            parentBone    = scene.errh_parent_bone
            row = box.row()
            row.prop(scene, 'errh_bone_collection_name', text="Collection")
            if toArmature != None and toArmature.type == 'ARMATURE':
                self._bone_search(box, scene, toArmature)
        # -----------------------------------------------------

        # -----------------------------------------------------
        if not using_rigify:
            if toArmature != None and toArmature.type == 'ARMATURE' and fromArmature != None and fromArmature.type == 'ARMATURE' and parentBone != '':
                if toArmature != fromArmature:
                    row = box.row()
                    row.operator(operator='scene.errh_add_bones_and_attach', text='Add bones and attach rig')
                    #row.enabled = True
        if using_rigify:
            if toArmature != None and toArmature.type == 'ARMATURE' and fromArmature != None and fromArmature.type == 'ARMATURE' and parentBone != '':
                if toArmature != fromArmature:
                    #row = box.row()
                    box.operator(operator='scene.errh_add_bones_to_armature', text='Add bones to meta rig')
                    #row = layout.row()
                    #box.separator()                
        # ----------------------------------------------------       
    
        box = layout.box()
        if using_rigify:
            if toArmature != None and toArmature.type == 'ARMATURE' and fromArmature != None and fromArmature.type == 'ARMATURE' and parentBone != '':
                box.label(text='To generated rig')
                box.prop(scene, 'errh_to_armature_generated', text="") 
                box.operator(operator='scene.errh_attach_rig', text='Attach \'from armature\' to generated rig')
            
            
            
            
            
            

def scene_poll_armature(self, object):
    return object.type == 'ARMATURE'
        
def register():
    bpy.utils.register_class(UI_PT_errh_panel)
    bpy.utils.register_class(errh_add_bones_and_attach)
    bpy.utils.register_class(errh_add_bones_to_armature)
    bpy.utils.register_class(errh_attach_rig)

    
    # pointer props
    bpy.types.Scene.errh_to_armature = bpy.props.PointerProperty(
        type=bpy.types.Object,
        name="To Armature",
        poll=scene_poll_armature,
        description="The armature to copy the lattice bones into (metarig if \'Use rigify mode\' is on)",
    )
    bpy.types.Scene.errh_to_armature_generated = bpy.props.PointerProperty(
        type=bpy.types.Object,
        name="Generated rig",
        poll=scene_poll_armature,
        description="The generated rig to use when searching for target bones generated from the \'From Armature\' (This is the rig the \'Lattice\' and \'From Armature\' objects will parented to. based on the currently selected \'Parent Bone\'",
    )
    bpy.types.Scene.errh_from_armature = bpy.props.PointerProperty(
        type=bpy.types.Object,
        name="From Armature",
        poll=scene_poll_armature,
        description="The armature to copy the lattice bones from",
    )
    # string props
    bpy.types.Scene.errh_parent_bone = bpy.props.StringProperty(
        name="Parent bone",
        description="The bone the new bones will parented to.",
    )
    bpy.types.Scene.errh_bone_collection_name = bpy.props.StringProperty(
        name="Bone collection name",
        description="Name of the collection to put the bones in.",
        default="External"
    )
    # bool
    bpy.types.Scene.errh_use_rigify_mode = bpy.props.BoolProperty(
        name="Use rigify mode",
        description="Allow adding bones to metarig and attaching bones to metarig target rig.",
    )
    
    
def unregister():
    bpy.utils.unregister_class(UI_PT_errh_panel)
    bpy.utils.unregister_class(errh_add_bones_and_attach)
    bpy.utils.unregister_class(errh_add_bones_to_armature)
    bpy.utils.unregister_class(errh_attach_rig)
    

    del bpy.types.Scene.errh_to_armature
    del bpy.types.Scene.errh_to_armature_generated
    del bpy.types.Scene.errh_from_armature
    del bpy.types.Scene.errh_parent_bone
    del bpy.types.Scene.errh_use_rigify_mode
    
if __name__ == '__main__':
    register()